@@ -1,5 +1,4 @@
 #!/usr/bin/perl
-
 eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
     if 0; # not running under some shell
 
@@ -166,13 +165,16 @@ END { if ($ENV{PAR_CLEAN}) {
     outs(qq{Removing files in "$par_temp"});
     File::Find::finddepth(sub { ( -d ) ? rmdir : unlink }, $par_temp);
     rmdir $par_temp;
-    rmdir $topdir;
+    # Don't remove topdir because this causes a race with other apps
+    # that are trying to start.
 
-    if (-d $par_temp) {
+    if (-d $par_temp && $^O ne 'MSWin32') {
         # Something went wrong unlinking the temporary directory.  This
         # typically happens on platforms that disallow unlinking shared
         # libraries and executables that are in use. Unlink with a background
         # shell command so the files are no longer in use by this process.
+        # Don't do anything on Windows because our parent process will
+        # take care of cleaning things up.
 
         my $tmp = new File::Temp(
             TEMPLATE => 'tmpXXXXX',
@@ -181,50 +183,23 @@ END { if ($ENV{PAR_CLEAN}) {
             UNLINK => 0,
         );
 
-        # Because the par_temp directory is going to be deleted in a
-        # background process, the parent process id may be reused before the
-        # background process completes.  To ensure that the temporary
-        # directory does not get reused while it is being deleted, try to
-        # rename it to a name that is related to the temporary script file.
-
-        my $tmpname = (File::Spec->splitpath($tmp->filename))[2]; # filename
-        $tmpname =~ s/\.[^.]*$/.dir/;
-        my $newDir = File::Spec->catpath(
-            (File::Spec->splitpath($par_temp))[0..1], $tmpname);
-        if (rename $par_temp, $newDir) {
-            outs("Renamed $par_temp to $newDir");
-            $par_temp = $newDir;
-        }
-
-        if ($^O =~ m/win32/i) {
-            print $tmp "
-:loop
-rmdir /q /s \"$par_temp\"
-if exist \"$par_temp\" goto loop
-rmdir \"$topdir\"
-rm \"" . $tmp->filename . "\"
-";
-            close $tmp;
-            my $proc;
-            Win32::Process::Create(
-                $proc, $ENV{COMSPEC},
-                "$ENV{COMSPEC} /c \"" . $tmp->filename . "\" >nul 2>nul ",
-                1, NORMAL_PRIORITY_CLASS, "."
-            );
-        } else {
-            print $tmp "#!/bin/sh
-while [ -d '$par_temp' ]; do
+        print $tmp "#!/bin/sh
+x=1; while [ \$x -lt 10 ]; do
    rm -rf '$par_temp'
+   if [ \! -d '$par_temp' ]; then
+       break
+   fi
+   sleep 1
+   x=`expr \$x + 1`
 done
-rmdir '$topdir'
 rm '" . $tmp->filename . "'
 ";
             chmod 0700,$tmp->filename;
-            my $cmd = $tmp->filename . ' >/dev/null 2>&1 &';
-            close $tmp;
-            system($cmd);
-        }
-        outs(qq(Spawned background process to perform cleanup));
+        my $cmd = $tmp->filename . ' >/dev/null 2>&1 &';
+        close $tmp;
+        system($cmd);
+        outs(qq(Spawned background process to perform cleanup: )
+             . $tmp->filename);
     }
 } }
 
@@ -402,8 +377,9 @@ my ($start_pos, $data_pos);
 my @par_args;
 my ($out, $bundle, $logfh, $cache_name);
 
-$quiet = 0 unless $ENV{PAR_DEBUG};
+delete $ENV{PAR_APP_REUSE}; # sanitize (REUSE may be a security problem)
 
+$quiet = 0 unless $ENV{PAR_DEBUG};
 # Don't swallow arguments for compiled executables without --par-options
 if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
     my %dist_cmd = qw(
@@ -414,51 +390,61 @@ if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
         v   verify_par
     );
 
-    my @add_to_inc;
-    while (@ARGV) {
-        $ARGV[0] =~ /^-([AIMOBLbqpiusTv])(.*)/ or last;
-
-        if ($1 eq 'I') {
-            push @add_to_inc, $2;
-        }
-        elsif ($1 eq 'M') {
-            eval "use $2";
-        }
-        elsif ($1 eq 'A') {
-            unshift @par_args, $2;
-        }
-        elsif ($1 eq 'O') {
-            $out = $2;
-        }
-        elsif ($1 eq 'b') {
-            $bundle = 'site';
-        }
-        elsif ($1 eq 'B') {
-            $bundle = 'all';
-        }
-        elsif ($1 eq 'q') {
-            $quiet = 1;
-        }
-        elsif ($1 eq 'L') {
-            open $logfh, ">>", $2 or die "XXX: Cannot open log: $!";
-        }
-        elsif ($1 eq 'T') {
-            $cache_name = $2;
-        }
-
-        shift(@ARGV);
-
-        if (my $cmd = $dist_cmd{$1}) {
-            delete $ENV{'PAR_TEMP'};
-            init_inc();
-            require PAR::Dist;
-            &{"PAR::Dist::$cmd"}() unless @ARGV;
-            &{"PAR::Dist::$cmd"}($_) for @ARGV;
-            exit;
-        }
+    # if the app is invoked as "appname --par-options --reuse PROGRAM @PROG_ARGV",
+    # use the app to run the given perl code instead of anything from the
+    # app itself (but still set up the normal app environment and @INC)
+    if (@ARGV and $ARGV[0] eq '--reuse') {
+        shift @ARGV;
+        $ENV{PAR_APP_REUSE} = shift @ARGV;
     }
+    else { # normal parl behaviour
 
-    unshift @INC, @add_to_inc;
+        my @add_to_inc;
+        while (@ARGV) {
+            $ARGV[0] =~ /^-([AIMOBLbqpiusTv])(.*)/ or last;
+
+            if ($1 eq 'I') {
+                push @add_to_inc, $2;
+            }
+            elsif ($1 eq 'M') {
+                eval "use $2";
+            }
+            elsif ($1 eq 'A') {
+                unshift @par_args, $2;
+            }
+            elsif ($1 eq 'O') {
+                $out = $2;
+            }
+            elsif ($1 eq 'b') {
+                $bundle = 'site';
+            }
+            elsif ($1 eq 'B') {
+                $bundle = 'all';
+            }
+            elsif ($1 eq 'q') {
+                $quiet = 1;
+            }
+            elsif ($1 eq 'L') {
+                open $logfh, ">>", $2 or die "XXX: Cannot open log: $!";
+            }
+            elsif ($1 eq 'T') {
+                $cache_name = $2;
+            }
+
+            shift(@ARGV);
+
+            if (my $cmd = $dist_cmd{$1}) {
+                delete $ENV{'PAR_TEMP'};
+                init_inc();
+                require PAR::Dist;
+                &{"PAR::Dist::$cmd"}() unless @ARGV;
+                &{"PAR::Dist::$cmd"}($_) for @ARGV;
+                exit;
+            }
+        }
+
+        unshift @INC, @add_to_inc;
+    }
 }
 
 # XXX -- add --par-debug support!
@@ -473,7 +459,6 @@ if ($out) {
         require Archive::Zip;
     }
 
-
     my $par = shift(@ARGV);
     my $zip;
 
@@ -540,6 +525,7 @@ if ($out) {
         require PAR::Heavy;
         PAR::Heavy::_init_dynaloader();
         init_inc();
+
         require_modules();
 
         my @inc = sort {
@@ -552,10 +538,13 @@ if ($out) {
              $_ ne $Config::Config{privlibexp});
         } @INC;
 
-        if ($Config{_delim} eq '\\') { s{\\}{/}g for @inc }
-
+        # File exists test added to fix RT #41790:
+        # Funny, non-existing entry in _<....auto/Compress/Raw/Zlib/autosplit.ix.
+        # This is a band-aid fix with no deeper grasp of the issue.
+        # Somebody please go through the pain of understanding what's happening,
+        # I failed. -- Steffen
         my %files;
-        /^_<(.+)$/ and $files{$1}++ for keys %::;
+        /^_<(.+)$/ and -e $1 and $files{$1}++ for keys %::;
         $files{$_}++ for values %INC;
 
         my $lib_ext = $Config::Config{lib_ext};
@@ -703,8 +692,9 @@ if ($out) {
             if (-f $dest_name && -s _ == $member->uncompressedSize()) {
                 outs(qq(Skipping "$member_name" since it already exists at "$dest_name"));
             } else {
-                $member->extractToFileNamed($dest_name);
                 outs(qq(Extracting "$member_name" to "$dest_name"));
+                $member->extractToFileNamed($dest_name);
+                chmod(0555, $dest_name) if $^O eq "hpux";
             }
         }
     }
@@ -744,6 +734,7 @@ sub require_modules {
     require vars;
     require Carp;
     require Carp::Heavy;
+    require Errno;
     require Exporter::Heavy;
     require Exporter;
     require Fcntl;
@@ -759,16 +750,19 @@ sub require_modules {
     require PAR::Heavy;
     require PAR::Dist;
     require PAR::Filter::PodStrip;
+    require attributes;
     eval { require Cwd };
     eval { require Win32 };
-    eval { require Win32::Process };
     eval { require Scalar::Util };
     eval { require Archive::Unzip::Burst };
+    eval { require Tie::Hash::NamedCapture };
+    eval { require PerlIO; require PerlIO::scalar };
 }
 
 # The C version of this code appears in myldr/mktmpdir.c
+# This code also lives in PAR::SetupTemp as set_par_temp_env!
 sub _set_par_temp {
-    if ($ENV{PAR_TEMP} and $ENV{PAR_TEMP} =~ /(.+)/) {
+    if (defined $ENV{PAR_TEMP} and $ENV{PAR_TEMP} =~ /(.+)/) {
         $par_temp = $1;
         return;
     }
@@ -777,7 +771,7 @@ sub _set_par_temp {
         (map $ENV{$_}, qw( PAR_TMPDIR TMPDIR TEMPDIR TEMP TMP )),
         qw( C:\\TEMP /tmp . )
     ) {
-        next unless $path and -d $path and -w $path;
+        next unless defined $path and -d $path and -w $path;
         my $username;
         my $pwuid;
         # does not work everywhere:
@@ -860,8 +854,9 @@ sub _tempfile {
     return($fh, $filename);
 }
 
+# same code lives in PAR::SetupProgname::set_progname
 sub _set_progname {
-    if ($ENV{PAR_PROGNAME} and $ENV{PAR_PROGNAME} =~ /(.+)/) {
+    if (defined $ENV{PAR_PROGNAME} and $ENV{PAR_PROGNAME} =~ /(.+)/) {
         $progname = $1;
     }
 
@@ -981,7 +976,8 @@ L<PAR>, L<PAR::Dist>, L<parl>, L<pp>
 
 =head1 AUTHORS
 
-Audrey Tang E<lt>cpan@audreyt.orgE<gt>
+Audrey Tang E<lt>cpan@audreyt.orgE<gt>,
+Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
 L<http://par.perl.org/> is the official PAR website.  You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
@@ -991,8 +987,7 @@ Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright 2002, 2003, 2004, 2005, 2006 by Audrey Tang
-E<lt>cpan@audreyt.orgE<gt>.
+Copyright 2002-2009 by Audrey Tang E<lt>cpan@audreyt.orgE<gt>.
 
 Neither this program nor the associated L<parl> program impose any
 licensing restrictions on files generated by their execution, in
